@font-face {
    font-family: EagerNaturalist;
    src: url("eager.eot?") format("eot"), url("eager.woff") format("woff"), url("eager.ttf") format("truetype"), url("eager.svg#EagerNaturalist") format("svg");
    font-weight: normal;
    font-style: normal;
}

body {
    font-family: EagerNaturalist, sans-serif;
    background-image: url("paper.jpg");
    background-repeat: repeat;
    color: #484849;
}

#cy {
    height: 100%;
    width: 100%;
    position: absolute;
    left: 0;
    top: 0;
}

#agp-panel {
    position: absolute;
    /*padding: 6px 15px 6px 15px;*/
    background-image: url('post-it.png');
    width: 250px;
    height: 250px;
    background-repeat: no-repeat;
}

#title {
    margin: 13px;
    text-align: center;
    font-size: 1.4em;
    -webkit-transform: rotate(2deg);
    -moz-transform: rotate(2deg);
    -o-transform: rotate(2deg);
}

#tps {
    font-family: EagerNaturalist, sans-serif;
    -webkit-transform: rotate(2deg);
    -moz-transform: rotate(2deg);
    -o-transform: rotate(2deg);
}

#brand {
    position: absolute;
    right: 0;
    bottom: 0;
    background-image: url("logo-sdh.png");
    z-index: -1;
    width: 200px;
    height: 79px;
    background-repeat: no-repeat;
    margin: 15px;
}

p {
    margin: 7px;
    font-size: 18px;
    padding-left: 10%;
}
